' STMPTERM.BAS - BASIC Stamp Terminal 
'
' Serial interface experiment for the Parallax BASIC Stamp
' Complimentary QBasic program is called QBTERM.BAS
'
' Jon Williams
' Riverside, CA
' CIS 73110,3272


' -----[ SYMBOLS ] --------------------------------------------------------
'
SYMBOL  S_In    = 7
SYMBOL  S_Out   = 6
SYMBOL  E       = 5                     ' LCD enable pin (1 = enabled)
SYMBOL  RS      = 4                     ' Register Select (0 = instruction)
'
' variables
'
SYMBOL  outp    = b0                    ' output workspace
SYMBOL  char    = b1                    ' char sent to LCD
SYMBOL  index1  = b2
SYMBOL  index2  = b3
'
' LCD control characters
'
SYMBOL  ClrLCD  = $01                   ' clear the LCD
SYMBOL  CrsrHm  = $02                   ' move cursor to home position
SYMBOL  CrsrLf  = $10                   ' move cursor left
SYMBOL  CrsrRt  = $14                   ' move cursor right
SYMBOL  DispLf  = $18                   ' shift displayed characters left
SYMBOL  DispRt  = $1C                   ' shift displayed characters right


' -----[ Here we go... ]---------------------------------------------------
'
Begin:  pins = %00000000
	dirs = %00111111
	pause 20                        ' let the LCD settle


' -----[ Initialize the LCD (Hitatchi HD44780 controller) ]----------------
'
Init:   pins = %00000011                ' 8-bit mode
	pulsout E, 1
	pause 5
	pulsout E, 1
	pause 1
	pulsout E, 1
	pins = %00000010                ' 4-bit mode
	pulsout E, 1
	char = %00001110                ' display on, cursor on, blink off
	gosub WrLCD
	char = %00000110                ' inc cursor, no disp shift
	gosub WrLCD
	char = %00000001                ' clear LCD
	gosub WrLCD
	high RS                         ' LCD to character mode


' -----[ Main program ]----------------------------------------------------
'
Main:   serin S_In, N2400, char
	if char > 13 then Out           ' send the character to LCD
	if char =  3 then Clear         ' clear the LCD, home the cursor
	if char =  8 then BkSpc         ' move the cursor back one pos
	if char = 13 then Home          ' home the cursor
	if char =  5 then Resp          ' respond to enquiry
	goto Main 

Out:    gosub WrLCD
	goto Main

Clear:  char = ClrLCD
	gosub LCDcmd
	goto Main

BkSpc:  char = CrsrLf
	gosub LCDcmd
	char = " "
	gosub WrLCD
	char = CrsrLf
	gosub LCDcmd
	goto Main

Home:   char = CrsrHm
	gosub LCDcmd
	goto Main

Resp:   serout S_Out, N2400, ("Stamp here")
	goto Main


' -----[ LCD Command ]-----------------------------------------------------
'
' Load char with command value, then call
'
'   Clear the LCD............. $01, %00000001
'   Home the cursor........... $02, %00000010
'   Display control........... (see below)
'   Entry mode................ (see below)
'   Cursor left............... $10, %00010000
'   Cursor right.............. $14, %00010100
'   Scroll display left....... $18, %00011000
'   Scroll display right...... $1C, %00011100
'
' Display control byte:
'
'   % 0 0 0 0 1 D C B
'                  blink character under cursor (1 = blink)
'                 cursor on/off (1 = on)
'                display on/off (1 = on)
'
' Entry mode byte:
'
'   % 0 0 0 0 0 1 X S
'                   shift display (S = 1), left (X = 1), right (X = 0)
'                  cursor move: right (X = 1), left (X = 0)
'
'
LCDcmd: low RS
	gosub WrLCD
	high RS
	return


' -----[ Write ASCII char to LCD ]-----------------------------------------
'
WrLCD:  pins = pins & %00010000         ' E = High, data bus clear
	outp = char / 16                ' get high nibble
	pins = pins | outp              ' output the nibble
	pulsout E, 1                    ' strobe the Enable line
	pins = pins & %00010000
	outp = char & %00001111         ' get low nibble
	pins = pins | outp
	pulsout E, 1
	return
